//Library for the temperature and humidity sensor
#include <DHT.h>

//Libraries to use the OLED screen
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
//Instanciation of the display
Adafruit_SSD1306 display(128, 64, &Wire, -1);

//Definition of the DHT sensor's signal pin
#define DHTpin 3  
DHT dht(DHTpin, DHT22);

//Definition of the control and enable pins of the H-bridge
#define pinIN1        9     
#define pinIN2        10       
#define pinIN3        11      
#define pinIN4        12       
#define pinENA        6
#define pinENB        5

//Definition of the limit switch pins
#define closedlimitswitch        7
#define openlimitswitch       8


//Définition of the variables
bool hatchOpen = false;
float humidityLevel = 0.0;
float temperature = 0.0;
float humidityThreshold = 52.0;

void setup() {
  
  // Configuration of the control and enable pins of the H-bridge
  pinMode(pinENA, OUTPUT);
  pinMode(pinIN1, OUTPUT);
  pinMode(pinIN2, OUTPUT);
  pinMode(pinIN3, OUTPUT);
  pinMode(pinIN4, OUTPUT);
  pinMode(pinENB, OUTPUT);

  // Configuration of the limit switch pins
  pinMode(openlimitswitch, INPUT);
  pinMode(closedlimitswitch, INPUT);

  // Begin the serial link for debugging
  Serial.begin(9600);

  //Initialization of the DHT sensor
  dht.begin();

  //Initialization of the OLED screen
  // Note that the address may be different, you can find the correct one by using the code i2c_scanner included in the wire.h library
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
  display.display();
  display.clearDisplay();
}

void loop() {

  //Get sensor readings
  humidityLevel = dht.readHumidity();             
  temperature = dht.readTemperature();

  //When the humidity level is too high, the fan starts and the hatch is opened
  if(humidityLevel>humidityThreshold && hatchOpen == false){
    updateScreen();
    startFan();
    openHatch();
    hatchOpen = true;
  }

  //When the humidity level is low enough, the hatch is closed and the fan stops
  if(humidityLevel<humidityThreshold && hatchOpen == true){
    updateScreen();
    closeHatch();
    stopFan();
    hatchOpen = false;
  }

  // Printing the sensor values and hatch status for debugging
  Serial.print("Humidity level = "); Serial.print(humidityLevel, 1); Serial.println(" %");
  Serial.print("Temperature = "); Serial.print(temperature, 1); Serial.println(" °C");
  Serial.print("Open hatch limit switch = "); Serial.println(hatchOpen);
  Serial.println();

  //Update the screen to the latest readings
  updateScreen();
  // Two second delay between humidity and temperature readings, per the manufacturer's instructions
  delay(2000);
}

void updateScreen(){
  display.clearDisplay();
  display.setTextSize(2);             // Text size 2
  display.setTextColor(WHITE);        // White font
  display.setCursor(0,0);   
  display.print(F("Humidity : ")); 
  display.print(int(humidityLevel));
  display.println(F("%"));
  display.print(F("Temp : "));
  display.print(int(temperature));
  display.println(F("C"));
  display.display();
}

//Function to start the fan
void startFan(){
  digitalWrite(pinENB, HIGH);
  digitalWrite(pinIN3, LOW);
  digitalWrite(pinIN4, HIGH);
}

//Function to stop the fan
void stopFan(){
  digitalWrite(pinENB, LOW);
  digitalWrite(pinIN3, LOW);
  digitalWrite(pinIN4, LOW);
}

//Fonction to open the hatch. The motor will not stop turning until the limit switch is activated.
void openHatch(){
  bool fdc = false;
  do{
    digitalWrite(pinENA, HIGH);
    digitalWrite(pinIN1, LOW);            
    digitalWrite(pinIN2, HIGH); 
    fdc = digitalRead(openlimitswitch);
    //Serial.println(fdc);
  }while(fdc == true);
  digitalWrite(pinENA, LOW);
  digitalWrite(pinIN2, LOW);
}

//Function to close the hatch in a similar fashion
void closeHatch(){
  bool fdc = false;
  do{
    digitalWrite(pinENA, HIGH);
    digitalWrite(pinIN1, HIGH);                
    digitalWrite(pinIN2, LOW);
    fdc = digitalRead(closedlimitswitch); 
    //Serial.println(fdc);
  }while(fdc == true);
  digitalWrite(pinENA, LOW);
  digitalWrite(pinIN1, LOW); 
}
